#!/bin/bash

  #===========================================================#
  #                                                           #
  #  Name: Patch_and_Install_VMware_Tools_for_Ubuntu_9_04.sh  #
  #  Written By: WoodyZ                                       #
  #  Created: 05-04-2009                                      #
  #  Version: 1.00.10                                         #
  #  Modified: 05-10-2009                                     #
  #                                                           #
  #  Notes:                                                   #
  #                                                           #
  #  This Script is a very basic command scrip written to     #
  #  simplify the patching and installing of VMware Tools     #
  #  in Ubuntu 9.04 and to also resolve a few issues in       #
  #  part due to Ubuntu 9.04 not being supported at the       #
  #  present time in VMware Fusion and to in short just       #
  #  make the entire process and various issues taken         #
  #  care of all at one time with the least amount of         #
  #  energy and effort especially for users that are          #
  #  not comfortable using the Terminal and a CLI.            #
  #                                                           #
  #  Change Log:                                              #
  #                                                           #
  #  1.00.10 Added conditional statements and error checking. #
  #                                                           #
  #  1.00.00 Original Release.                                #
  #                                                           #
  #===========================================================#



    # [Error Functions]
    #
    # These are routines that can be called
    # more then once throughout the script.


    # Notifies User there was an error creating
    # the patch file and then terminate script.

function Patch_File_Missing () {
  clear
  echo
  echo '  Error creating the '$1' file!'
  echo
  echo '  Restart shell script, try again...'
  echo  
  echo '  Press Enter to close this window...'
  echo $'\a'  # Ring System Bell.
  read
  exit 1
}

    # Traps failed sudo command and terminates script.

function Strike_Three_Your_Out () {
  clear
  echo
  echo '  Incorrect password to many times!'
  echo
  echo '  Restart shell script, try again...'
  echo  
  echo '  Press Enter to close this window...'
  echo $'\a'  # Ring System Bell.
  read
  exit 1
}


    # [Main Function]


    # This Function is called if all preflight conditions are meet.
    #    
    # See Preflight Conditions Check comments for what must be met.


function Patch_and_Install_VMware_Tools () {

    # Inform the User what this scrip will do.

echo
echo '  This script will resolve several issues with Ubuntu 9.04 Desktop Edition'
echo '  run under VMware Fusion 2.0.4 by downloading and installing a few packages'
echo '  needed to resolve mouse issues and patch VMware Tools prior to installing'
echo '  them and also patch the fstab file to enable the User to access VMware'
echo '  Shared Folders after the patch to VMware Tools. The HGFS Share issue and '
echo '  fstab issue are separate issues yet has to be resolved on Linux installs.'
echo
echo '  You will be prompted for your Password at least twice, once at the start'
echo '  to download/install the needed packages then again to install VMware Tools '
echo '  and patch the fstab file.  If VMware Tools are already installed you will'
echo '  also be prompted for uninstalling and automatic reboot if you so choose.'
echo
echo '  Do not forget to save the output of this script when finished and there'
echo '  are instruction at the end of the process that to explain how to do it.'

    # Install diff-doc, mdtect, patch and xserver-xorg-input-vmmouse.

echo
echo
echo  '  [Update Packages and Patch VMware Tools]'
echo
echo  '  Installing diff-doc, mdtect, patch and xserver-xorg-input-vmmouse...'
echo
echo  '  Type your Password then press Enter...'
echo
sudo apt-get install diff-doc mdetect patch xserver-xorg-input-vmmouse

    # Trap failed sudo.

if [ $? -gt 0 ]; then Strike_Three_Your_Out; fi

    # Run as normal User until ready to install VMware Tools.

sudo -k 

    # Check to see the patch executable is installed.
    #
    # While patch should have been installed via apt-get
    # this is to ensure it is and located where expected.

if [ ! -e '/usr/bin/patch' ]; then
  clear
  echo  
  echo '  The patch executable /usr/bin/patch does not exist!'
  echo
  echo '  This program is needed to patch VMware Tools and /etc/fstab file.'
  echo '  Resolve this dependency and then execute this shell script again.'
  echo
  echo  $'\a'  # Ring System Bell.
  echo '  Press Enter to close this window...'
  read
  exit 1
fi

    # Prepare to patch and install VMware Tools.

cd /tmp
echo
echo  '  Preparing/Patching VMware Tools...'
echo

    # Make sure vmware-tools-distrib
    # folder does not already exist.

if [ -d 'vmware-tools-distrib' ]; then
  rm -dfr vmware-tools-distrib
fi

    # Extract VMware Tools from mounted linux.iso file.

tar zxpf /media/cdrom$CD_NUMBER/*.tar.gz

    # Extract the vmhgfs.tar tarball to patch the page.c file.

cd /tmp/vmware-tools-distrib/lib/modules/source
tar xpf vmhgfs.tar

    # Backup the original vmhgfs.tar tarball.

echo '  Backing up vmhgfs.tar to vmhgfs-original.tar...'
mv vmhgfs.tar vmhgfs-original.tar

    # Create the patch file for the page.c file.

echo '867c867' > page.c.patch
echo '<    page = __grab_cache_page(mapping, index);' >> page.c.patch
echo '---' >> page.c.patch
echo '>    page = grab_cache_page_write_begin(mapping, index, flags);' >> page.c.patch

    # Verify the the page.c.patch file exists.

if [ ! -e 'page.c.patch' ]; then
  Patch_File_Missing 'page.c.patch'
fi

    # Patch the page.c file.

echo  '  Patching page.c file...'
patch ./vmhgfs-only/page.c page.c.patch > /dev/null 2>&1

    # Clean up: Delete the page.c.patch file.

rm page.c.patch

    # Create the new vmhgfs.tar tarball.

echo  '  Creating new vmhgfs.tar tarball...'
tar cvf vmhgfs.tar vmhgfs-only > /dev/null 2>&1

    # Set permissions on the vmhgfs.tar tarball to Read Only.

chmod 444 vmhgfs.tar

    # Clean up: Delete the vmhgfs-only folder.

rm -dfr vmhgfs-only

    # Create the patch for the /etc/fstab file.

echo '0c0' > /tmp/fstab.patch
echo '< .host:/                 /mnt/hgfs               vmhgfs  defaults,ttl=5     0 0' >> /tmp/fstab.patch
echo '---' >> /tmp/fstab.patch
echo '> .host:/  /mnt/hgfs  vmhgfs  defaults,ttl=5,uid='${UID}',gid='${UID}'  0 0' >> /tmp/fstab.patch

    # Verify that the pfstab.patch file exists.

if [ ! -e '/tmp/fstab.patch' ]; then
  Patch_File_Missing 'fstab.patch'
fi

    # Install the patched VMware Tools and patch fstab.

cd /tmp/vmware-tools-distrib
echo
echo  '  [Install VMware Tools and Patch /etc/fstab File]'
echo
echo  '  Installing VMware Tools and patching /etc/fstab...'
echo
echo  '  Type your Password then press Enter...'
echo $'\a'  # Ring System Bell.
sudo ./vmware-install.pl

    # Trap failed sudo.

if [ $? -gt 0 ]; then Strike_Three_Your_Out; fi

    # Backup the /etc/fstab file before patching it.

echo
echo  '  Backing up /etc/fstab to /etc/fstab.backup...'
sudo cp /etc/fstab /etc/fstab.backup


    # Patch the /etc/fstab file.

echo  '  Patching /etc/fstab...'
sudo patch /etc/fstab /tmp/fstab.patch > /dev/null 2>&1

    # Clean up: Delete the /tmp/fstab.patch file.

rm /tmp/fstab.patch

    # umount/mount HGFS Filesystem.

echo '  Unmounting HGFS Filesystem...'
sudo umount /mnt/hgfs
sleep 2s
echo '  Mounting HGFS Filesystem...'
sudo mount .host:/ /mnt/hgfs -t vmhgfs -o defaults,ttl=5,uid=${UID},gid=${UID}

    # Run as normal User until ready to reboot to avoid
    # accidentally rebooting the system before saving output.

sudo -k

    # Inform User the process is complete and to save the output.

echo
echo
echo '  Installation Process Complete!...'
echo
echo '  Save the output of this shell script for your records. Now copy and paste'
echo
echo '  the results via Edit > Select All and then Edit > Copy (Shift+Ctrl+C) to'
echo
echo '  place the information on the Clipboard to then paste it into Text Editor.'
echo
echo '  Click Applications > Accessories > Text Editor where you can then paste'
echo
echo '  (Ctrl+V) the information and save the file.  After saving press Enter'
echo
echo '  to automatically reboot to allow all changes to take place or to'
echo
echo '  manually reboot you can close this window by the X then reboot.'
echo $'\a'  # Ring System Bell.
read
sudo shutdown -r now 'System is rebooting...'
exit 0

}


    # Preflight Conditions Check 1 of 3...
    #
    # Check to see if VMware Tools are already installed.
    #
    # If VMware Tools are installed offer to automatically
    # remove and reboot or User manually remove and reboot.

echo
echo '  Checking if VMware Tools are installed...'
if [ -e '/usr/bin/vmware-uninstall-tools.pl' ]; then

      # VMware Tools are already installed!

  clear
  echo $'\a'  # Ring System Bell.
  echo '   VMware Tools are already installed!'
  echo ' ---------------------------------------'
  echo
  echo '       What would you like to do?' 
  echo
  echo '  [1] Automatically Uninstall & Reboot?'
  echo
  echo '  [2] Exit Script & Manually Uninstall?'
  echo
  echo '  Press 1 or 2 then press Enter...'
  echo
  read key
  echo

    case "$key" in

      "1")
        echo '  Authorize uninstalling VMware Tools and Reboot...'
        echo
        sudo /usr/bin/vmware-uninstall-tools.pl

              # Trap failed sudo.

          if [ $? -gt 0 ]; then Strike_Three_Your_Out; fi

                # Uninstall complete. Inform User to save output.

        echo
        echo
        echo '  If you want to save the output of the uninstall command then copy and paste'
        echo
        echo '  the results via Edit > Select All and then Edit > Copy (Shift+Ctrl+C) to'
        echo
        echo '  place the information on the Clipboard to then paste it into Text Editor.'
        echo
        echo '  Click Applications > Accessories > Text Editor where you can then paste'
        echo
        echo '  (Ctrl+V) the information and save the file for your records'
        echo
        echo
        echo '  Then press Enter and system will automatically reboot...'
        echo $'\a'  # Ring System Bell.
        read
        sudo shutdown -r now 'System is rebooting...'
        exit 0
      ;;

                # User chose to manually uninstall VMware Tools and reboot.

      "2")
        echo '  You have chosen to manually uninstall VMware Tools.  Open a'
        echo '  Terminal and type the following command then press Enter...'
        echo
        echo '  sudo /usr/bin/vmware-uninstall-tools.pl'
        echo
        echo '  Then reboot the system after uninstalling VMware Tools'
        echo '  and then execute this shell script again...'
        echo $'\a'  # Ring System Bell.
        echo '  Press Enter to close this window...'
        read
        exit 1
      ;;

                # Really now, how hard is it to press 1 or 2?!

      *)
        echo "  " '"'$key'"' "is not a valid choice... Try again!"
        echo
        echo $'\a'  # Ring System Bell.
        read -p " Press Enter to continue..."
        "$0"    # Run the shell script again.
      ;;

    esac

else

      # VMware Tools are not installed.

  echo '  VMware Tools are not installed...'

fi


    # Preflight Conditions Check 2 of 3...
    #
    # VMware Tools are not installed so check that system
    # has Internet Connectivity and if not then inform User
    # and then terminate the shell script.


    # Check Internet Connectivity.

echo '  Checking for Internet Connectivity...'
ping -c 1 ubuntu.com > /dev/null 2>&1

if [ ! $? -eq 0 ]; then

        # The Guest does not have Internet Connectivity.

    echo
    echo '  No Internet Connectivity Detected!...'
    echo
    echo '  Internet Connectivity is required!...'
    echo
    echo '  Connect then execute script again.'
    echo
    echo
    echo '  Press Enter to close this window...'
    echo $'\a'  # Ring System Bell.
    read
    exit 1

else

      # Guest has Internet Connectivity.

  echo  '  Guest has Internet Connectivity...'

fi


    # Preflight Conditions Check 3 of 3...
    #
    # Check to see if VMware Tools linux.iso is mounted and
    # check for three CD/DVD's and which one is it mounted in.
    #
    # This will handle the normal and Linux Easy Install defaults
    # and just to be safe three total just in case User added 1 more.


echo '  Checking for VMware Tools Disc...'
for i in 2 1 0
do
  if [ -d /media/cdrom$i ]; then
    ls /media/cdrom$i | grep VMwareTools > /dev/null 2>&1
    if [ $? -eq 0 ]; then
          # VMware Tools is mounted in cdrom$i
      CD_NUMBER=$i
      echo '  VMware Tools mounted in cdrom'$CD_NUMBER'...'
          # All preflight conditions checked out!
          # Patch and install VMware Tools and fstab.
      Patch_and_Install_VMware_Tools
    else
      VMTOOLS_CD='1'
    fi
  else
    VMTOOLS_CD='1'
  fi
done

if [ $VMTOOLS_CD = 1 ]; then

      # The VMware Tools Disc is not mounted.
  echo
  echo
  echo '  The VMware Tools installation disc is not mounted!...'
  echo
  echo '  Click Install VMware Tools on the Virtual Machine'
  echo '  menu and then execute this shell script again.'
  echo
  echo
  echo '  Press Enter to close this window...'
  echo $'\a'  # Ring System Bell.
  read
  exit 1
fi

# END.

    # Note: The Preflight Conditions Checks could have been
    # nestled together however it was easier not to and not
    # really needed to be done and would probably be easier
    # for someone to modify the shell script if not nestled.

###

